#!/usr/bin/python
# -*- coding: utf-8 -*-
# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

import brlapi
from fenrirscreenreader.core import debug

class brailleDriver():
    def __init__(self):
        self._isInitialized = False
        self._brl = None
        self.deviceSize = None

    def initialize(self, environment):
        """Initialize the BRLTTY connection."""
        self.env = environment
        try:
            self._brl = brlapi.Connection()
            self._brl.enterTtyMode()
            self.deviceSize = self._brl.displaySize
            self._isInitialized = True
            return True
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('ERROR: Initializing braille failed:' + str(e),debug.debugLevel.ERROR)
            return False

    def getDeviceSize(self):
        """Get the size of the braille display."""
        if not self._isInitialized:
            return (0, 0)
        return self.deviceSize if self.deviceSize else (0, 0)

    def writeText(self, text):
        """Write text to the braille display."""
        if not self._isInitialized:
            return
        try:
            self._brl.writeText(text)
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('ERROR: Writing braille failed:' + str(e),debug.debugLevel.ERROR)

    def readKeypress(self):
        """Read a keypress from the braille display."""
        if not self._isInitialized:
            return None
        try:
            return self._brl.readKey(wait=0)
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('ERROR: Reading key failed:' + str(e),debug.debugLevel.ERROR)
            return None

    def enterScreen(self, screen):
        """Enter a new screen context."""
        if not self._isInitialized:
            return
        try:
            self._brl.enterTtyModeWithPath(screen)
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('ERROR: Entering screen failed:' + str(e),debug.debugLevel.ERROR)

    def leaveScreen(self):
        """Leave the current screen context."""
        if not self._isInitialized:
            return
        try:
            self._brl.leaveTtyMode()
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('ERROR: Leaving screen failed:' + str(e),debug.debugLevel.ERROR)

    def shutdown(self):
        """Shutdown the braille driver."""
        if not self._isInitialized:
            return
        try:
            self.leaveScreen()
            if self._brl:
                self._brl.closeConnection()
            self._brl = None
            self._isInitialized = False
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('ERROR: Shutting down braille failed:' + str(e),debug.debugLevel.ERROR)
